@@ -1,51 +1,54 @@
 #!/usr/bin/python3
-# -*- coding: utf-8 -*-
 
-""" This file is licensed under GPLv3, see https://www.gnu.org/licenses/ """
+"""Licensed under GPLv3, see https://www.gnu.org/licenses/"""
 
-import os
-import sys
-import readline
-import signal
-import codecs
-import shutil
 import atexit
+import codecs
 import io
+import os
+import readline
+import shutil
+import signal
+import sys
 from argparse import ArgumentError
-from typing import Optional, Callable
+from contextlib import AbstractContextManager
+from typing import Any, Callable
 
 import pyalpm
 
-from .i18n import translate  # keep that first
-from .args import (
-    parse_args, reconstruct_args,
+from .args import parse_args
+from .config import (
+    _OLD_AUR_REPOS_CACHE_PATH,
+    AUR_REPOS_CACHE_PATH,
+    CACHE_ROOT,
+    CONFIG_PATH,
+    DATA_ROOT,
+    PikaurConfig,
 )
-from .help_cli import cli_print_help
 from .core import (
     DEFAULT_INPUT_ENCODING,
-    spawn, interactive_spawn, remove_dir, check_runtime_deps,
-    running_as_root, sudo, isolate_root_cmd, run_with_sudo_loop,
-)
-from .pprint import (
-    color_line, bold_line,
-    print_stderr, print_stdout,
-    print_error, print_warning,
-    create_debug_logger, ColorsHighlight,
-)
-from .print_department import print_version
-from .updates import print_upgradeable
-from .prompt import ask_to_continue, get_multiple_numbers_input, NotANumberInput
-from .config import (
-    BUILD_CACHE_PATH, PACKAGE_CACHE_PATH, CACHE_ROOT, CONFIG_PATH,
-    AUR_REPOS_CACHE_PATH, PikaurConfig, _OLD_AUR_REPOS_CACHE_PATH, DATA_ROOT,
+    check_runtime_deps,
+    interactive_spawn,
+    isolate_root_cmd,
+    run_with_sudo_loop,
+    running_as_root,
+    spawn,
+    sudo,
 )
 from .exceptions import SysExit
-from .pikspect import TTYRestore, PikspectSignalHandler
-from .install_cli import InstallPackagesCLI
-from .search_cli import cli_search_packages
-from .info_cli import cli_info_packages
-from .urllib import init_proxy, ProxyInitSocks5Error
 from .getpkgbuild_cli import cli_getpkgbuild
+from .help_cli import cli_print_help
+from .i18n import translate
+from .info_cli import cli_info_packages
+from .install_cli import InstallPackagesCLI
+from .pikspect import PikspectSignalHandler, TTYRestore
+from .pkg_cache_cli import cli_clean_packages_cache
+from .pprint import create_debug_logger, print_error, print_stderr, print_warning
+from .print_department import print_version
+from .prompt import NotANumberInputError, get_multiple_numbers_input
+from .search_cli import cli_search_packages
+from .updates import print_upgradeable
+from .urllib import ProxyInitSocks5Error, init_proxy
 
 
 def init_readline() -> None:
@@ -63,24 +66,42 @@ init_readline()
 _debug = create_debug_logger('main')
 
 
-def init_output_encoding() -> None:
-    for attr in ('stdout', 'stderr'):
-        real_stream = getattr(sys, attr)
-        try:
+class OutputEncodingWrapper():
+
+    original_stderr: int
+    original_stdout: int
+
+    def __enter__(self) -> None:
+        for attr in ('stdout', 'stderr'):
+            _debug(f'Setting {attr} to unicode...')
+            real_stream = getattr(sys, attr)
+            try:
+                setattr(
+                    self, f'original_{attr}',
+                    real_stream
+                )
+                setattr(
+                    sys, attr,
+                    codecs.open(
+                        real_stream.fileno(),
+                        mode='w', buffering=0, encoding=DEFAULT_INPUT_ENCODING
+                    )
+                )
+            except io.UnsupportedOperation:
+                pass
+            else:
+                getattr(sys, attr).buffer = real_stream.buffer
+
+    def __exit__(self, *_exc_details: Any) -> None:
+        for attr in ('stdout', 'stderr'):
+            _debug(f'Restoring {attr}...', lock=False)
+            stream = getattr(sys, attr)
+            orig_stream = getattr(self, f'original_{attr}')
+            stream.close()
             setattr(
                 sys, attr,
-                codecs.open(  # pylint: disable=consider-using-with
-                    real_stream.fileno(),
-                    mode='w', buffering=0, encoding=DEFAULT_INPUT_ENCODING
-                )
+                orig_stream
             )
-        except io.UnsupportedOperation:
-            pass
-        else:
-            getattr(sys, attr).buffer = real_stream.buffer
-
-
-init_output_encoding()
 
 
 def cli_print_upgradeable() -> None:
@@ -95,28 +116,6 @@ def cli_pkgbuild() -> None:
     cli_install_packages()
 
 
-def cli_clean_packages_cache() -> None:
-    args = parse_args()
-    if not args.repo:
-        for directory, message, minimal_clean_level in (
-                (BUILD_CACHE_PATH, translate("Build directory"), 1, ),
-                (PACKAGE_CACHE_PATH, translate("Packages directory"), 2, ),
-        ):
-            if minimal_clean_level <= args.clean and os.path.exists(directory):
-                print_stdout(f"\n{message}: {directory}")
-                if ask_to_continue(text='{} {}'.format(  # pylint: disable=consider-using-f-string
-                        color_line('::', ColorsHighlight.blue),
-                        bold_line(translate("Do you want to remove all files?"))
-                )):
-                    remove_dir(directory)
-    if not args.aur:
-        raise SysExit(
-            interactive_spawn(sudo(
-                [PikaurConfig().misc.PacmanPath.get_str(), ] + reconstruct_args(args)
-            )).returncode
-        )
-
-
 def cli_print_version() -> None:
     args = parse_args()
     pacman_version = spawn(
@@ -154,7 +153,7 @@ def cli_dynamic_select() -> None:  # pragma: no cover
             if restart_prompt:
                 continue
             break
-        except NotANumberInput as exc:
+        except NotANumberInputError as exc:
             if exc.character.lower() == translate('n'):
                 raise SysExit(128) from exc
             print_error(translate('invalid number: {}').format(exc.character))
@@ -177,7 +176,7 @@ def cli_entry_point() -> None:  # pylint: disable=too-many-statements
     # specified both operations, like `pikaur -QS smth`
 
     args = parse_args()
-    pikaur_operation: Optional[Callable] = None
+    pikaur_operation: Callable | None = None
     require_sudo = False
 
     if args.help:
@@ -299,39 +298,53 @@ def restore_tty() -> None:
     TTYRestore.restore()
 
 
-def handle_sig_int(*_whatever) -> None:  # pragma: no cover
+def handle_sig_int(*_whatever: Any) -> None:  # pragma: no cover
     if signal_handler := PikspectSignalHandler.get():
-        return signal_handler(*_whatever)  # pylint: disable=not-callable
+        signal_handler(*_whatever)  # pylint: disable=not-callable
+        return
     if parse_args().pikaur_debug:
         raise KeyboardInterrupt()
     print_stderr("\n\nCanceled by user (SIGINT)", lock=False)
     raise SysExit(125)
 
 
-def main() -> None:
-    try:
-        parse_args()
-    except ArgumentError as exc:
-        print_stderr(exc)
-        sys.exit(22)
-    check_runtime_deps()
+class EmptyWrapper:
 
-    create_dirs()
-    # initialize config to avoid race condition in threads:
-    PikaurConfig.get_config()
+    def __enter__(self) -> None:
+        pass
 
-    atexit.register(restore_tty)
-    signal.signal(signal.SIGPIPE, signal.SIG_DFL)
-    signal.signal(signal.SIGINT, handle_sig_int)
+    def __exit__(self, *_exc_details: Any) -> None:
+        pass
 
-    try:
-        cli_entry_point()
-    except BrokenPipeError:
-        # @TODO: should it be 32?
+
+def main(embed: bool = False) -> None:
+    wrapper: type[AbstractContextManager] = OutputEncodingWrapper
+    if embed:
+        wrapper = EmptyWrapper
+    with wrapper():
+        try:
+            parse_args()
+        except ArgumentError as exc:
+            print_stderr(exc)
+            sys.exit(22)
+        check_runtime_deps()
+
+        create_dirs()
+        # initialize config to avoid race condition in threads:
+        PikaurConfig.get_config()
+
+        atexit.register(restore_tty)
+        signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+        signal.signal(signal.SIGINT, handle_sig_int)
+
+        try:
+            cli_entry_point()
+        except BrokenPipeError:
+            # @TODO: should it be 32?
+            sys.exit(0)
+        except SysExit as exc:
+            sys.exit(exc.code)
         sys.exit(0)
-    except SysExit as exc:
-        sys.exit(exc.code)
-    sys.exit(0)
 
 
 if __name__ == '__main__':
